require 'lunatest'

local raygen = require( 'saotrace.raygen' )
local extent = require( 'saotrace.raygen.extent' )
local units  = require( 'saotrace.raygen.units' )
local convert = units.convert

local pos = { theta = { 20, 'arcmin' }, phi = { 30, 'degree' } , z = 390 }

function test_disk ()

   local ok, obj = pcall( extent.disk,
			   { name = 'a',
			     radius = { 10, 'arcsec' },
			     position = pos,
			     idist = { 'gaussian', sigma = { 22, 'arcsec' } }
			  } )

   assert_true( ok, obj )

   local args = obj.args
   assert_equal( 'a', args.name )
   assert_equal( 10, convert( args.radius, nil, units.angular.arcsecond)[1] )
   assert_equal( 20, convert( args.position.theta, nil, units.angular.arcminute)[1] )
   assert_equal( 30, convert( args.position.phi, nil, units.angular.degree)[1], 1e-6 )
   assert_equal( 390, args.position.z[1] )
   assert_equal( 22, convert( args.idist.sigma, nil, units.angular.arcsecond)[1], 1e-6 )

   -- bogus idist
   local ok, obj = pcall( extent.disk,
			   { name = 'a',
			     radius = { 10, 'arcsec' },
			     position = pos,
			     idist = { 'furity' }
			  } )

   assert_false( ok )
   assert_match( 'unknown intensity distribution', obj  )

end

function test_rect ()

   local ok, obj = pcall( extent.rect,
			   { name = 'c',
			     height = { 10, 'arcsec' },
			     width  = { 11, 'arcsec' },
			     position = pos,
			     idist = { 'gaussian', sigma = { 22, 'arcsec' } }
			  })

   assert_true( ok, obj )

   local args = obj.args
   assert_equal( 'c', args.name )
   assert_equal( 10, convert( args.height, nil, units.angular.arcsecond)[1] )
   assert_equal( 11, convert( args.width, nil, units.angular.arcsecond)[1] )
   assert_equal( 20, convert( args.position.theta, nil, units.angular.arcminute)[1] )
   assert_equal( 30, convert( args.position.phi, nil, units.angular.degree)[1], 1e-6 )
   assert_equal( 390, args.position.z[1] )
   assert_equal( 22, convert( args.idist.sigma, nil, units.angular.arcsecond)[1], 1e-6 )


   -- bogus idist
   local ok, obj = pcall( extent.rect,
			   { name = 'a',
			     height = { 10, 'arcsec' },
			     width  = { 20, 'arcsec' },
			     position = pos,
			     idist = { 'furity'},
			  })

   assert_false( ok )
   assert_match( 'unknown intensity distribution', obj  )

end

function test_point ()

   local ok, obj = pcall( extent.point,
			   { name = 'b',
			     position = pos
			  } )

   assert_true( ok, obj )

   local args = obj.args
   assert_equal( 'b', args.name )
   assert_equal( 20, convert( args.position.theta, nil, units.angular.arcminute)[1] )
   assert_equal( 30, convert( args.position.phi, nil, units.angular.degree)[1], 1e-6 )
   assert_equal( 390, args.position.z[1] )

end

function test_image_pixsz()

   local ok, obj = pcall( extent.image,
			  {
			     name = 'a',
			     file = 'foo',
			     pixsz = 0.1,
			  } )

   assert_true( ok, obj )
   local args = obj.args
   assert_equal( 'a', args.name )
   assert_equal( 0.1, convert(args.pixsz, nil, units.angular.arcsecond)[1], 1e-6 )


   local ok, obj = pcall( extent.image,
			  {
			     name = 'a',
			     file = 'foo',
			     pixsz = { 10, 'arcminute' } ,
			  } )

   assert_true( ok, obj )
   local args = obj.args
   assert_equal( 10, convert(args.pixsz, nil, units.angular.arcminute)[1], 1e-6 )


   local ok, obj = pcall( extent.image,
			  {
			     name = 'a',
			     file = 'foo',
			     pixsz = { x = 10, y = 20 }
			  } )

   assert_true( ok, obj )
   local args = obj.args
   assert_equal( 10, convert(args.pixsz.x, nil, units.angular.arcsecond)[1], 1e-6 )
   assert_equal( 20, convert(args.pixsz.y, nil, units.angular.arcsecond)[1], 1e-6 )

end


lunatest.run()
